@@ -3,12 +3,14 @@ import warnings
 from typing import Any, Dict, List
 from lark import Lark, Transformer
 
+from .diagnostics import LIN_MASTER_REQUEST_FRAME_ID, LIN_SLAVE_RESPONSE_FRAME_ID, LinDiagnosticFrame, LinDiagnosticRequest, LinDiagnosticResponse
+from .schedule import AssignFrameIdEntry, AssignFrameIdRangeEntry, AssignNadEntry, ConditionalChangeNadEntry, DataDumpEntry, FreeFormatEntry, MasterRequestEntry, SaveConfigurationEntry, ScheduleTable, SlaveResponseEntry, UnassignFrameIdEntry, LinFrameEntry
+
 from .frame import LinEventTriggeredFrame, LinUnconditionalFrame
 from .signal import LinSignal
 from .encoding import ASCIIValue, BCDValue, LinSignalEncodingType, LogicalValue, PhysicalValue, ValueConverter
 from .lin import LIN_VERSION_2_0, LIN_VERSION_2_1, LinVersion
 from .node import LinMaster, LinProductId, LinSlave
-from .comment import parse_comments
 from .ldf import LDF
 
 def parse_ldf_to_dict(path: str, capture_comments: bool = False, encoding: str = None) -> Dict:
@@ -20,13 +22,18 @@ def parse_ldf_to_dict(path: str, capture_comments: bool = False, encoding: str =
     :param encoding: File encoding, for example 'UTF-8'
     :type encoding: str
     """
+    comments = []
     lark = os.path.join(os.path.dirname(__file__), 'lark', 'ldf.lark')
-    parser = Lark(grammar=open(lark), parser='lalr')
+    parser = Lark(grammar=open(lark), parser='lalr', lexer_callbacks={
+        'C_COMMENT': comments.append,
+        'CPP_COMMENT': comments.append
+    })
     ldf_file = open(path, "r", encoding=encoding).read()
     tree = parser.parse(ldf_file)
     json = LDFTransformer().transform(tree)
+
     if capture_comments:
-        json['comments'] = parse_comments(ldf_file)
+        json['comments'] = [comment.value for comment in comments]
     return json
 
 def parseLDFtoDict(path: str, captureComments: bool = False, encoding: str = None) -> Dict:
@@ -56,14 +63,18 @@ def parse_ldf(path: str, capture_comments: bool = False, encoding: str = None) -
     _populate_ldf_signals(json, ldf)
     _populate_ldf_frames(json, ldf)
     _populate_ldf_event_triggered_frames(json, ldf)
+    _populate_diagnostic_signals(json, ldf)
+    _populate_diagnostic_frames(json, ldf)
     _populate_ldf_nodes(json, ldf)
     _populate_ldf_encoding_types(json, ldf)
+    _populate_schedule_tables(json, ldf)
 
     _link_ldf_signals(json, ldf)
     _link_ldf_frames(json, ldf)
+    _link_ldf_schedule_table(json, ldf)
 
     if capture_comments:
-        ldf.comments = json['comments']
+        ldf._comments = json['comments']
 
     return ldf
 
@@ -157,6 +168,104 @@ def _create_ldf2x_node(node: dict, language_version: float):
 
     return slave
 
+def _populate_diagnostic_signals(json: dict, ldf: LDF):
+    if 'diagnostic_signals' in json:
+        for signal in json['diagnostic_signals']:
+            ldf._diagnostic_signals[signal['name']] = LinSignal.create(signal['name'], signal['width'], signal['init_value'])
+
+def _populate_diagnostic_frames(json: dict, ldf: LDF):
+    if 'diagnostic_frames' in json:
+        for frame in json['diagnostic_frames']:
+            signals = {}
+
+            for signal in frame['signals']:
+                s = ldf.get_diagnostic_signal(signal['signal'])
+                if s is None:
+                    raise ValueError(f"{frame['name']} references non existing signal {signal['signal']}")
+                signals[signal['offset']] = s
+
+            frame_obj = LinDiagnosticFrame(frame['frame_id'], frame['name'], 9, signals)
+            ldf._diagnostic_frames[frame['name']] = frame_obj
+            if frame['frame_id'] == LIN_MASTER_REQUEST_FRAME_ID:
+                ldf._master_request_frame = LinDiagnosticRequest(frame_obj)
+            if frame['frame_id'] == LIN_SLAVE_RESPONSE_FRAME_ID:
+                ldf._slave_response_frame = LinDiagnosticResponse(frame_obj)
+
+def _create_schedule_table_entry(json: dict, ldf: LDF):  # noqa: C901
+    if json['command']['type'] == 'frame':
+        entry = LinFrameEntry()
+        entry.delay = json['delay']
+        entry.frame = ldf.get_frame(json['command']['frame'])
+        return entry
+    if json['command']['type'] == 'master_request':
+        entry = MasterRequestEntry()
+        entry.delay = json['delay']
+        return entry
+    if json['command']['type'] == 'slave_response':
+        entry = SlaveResponseEntry()
+        entry.delay = json['delay']
+        return entry
+    if json['command']['type'] == 'assign_nad':
+        entry = AssignNadEntry()
+        entry.delay = json['delay']
+        entry.node = ldf.get_slave(json['command']['node'])
+        return entry
+    if json['command']['type'] == 'conditional_change_nad':
+        entry = ConditionalChangeNadEntry()
+        entry.delay = json['delay']
+        entry.nad = json['command']['nad']
+        entry.id = json['command']['id']
+        entry.byte = json['command']['byte']
+        entry.mask = json['command']['mask']
+        entry.inv = json['command']['inv']
+        entry.new_nad = json['command']['new_nad']
+        return entry
+    if json['command']['type'] == 'data_dump':
+        entry = DataDumpEntry()
+        entry.delay = json['delay']
+        entry.node = ldf.get_slave(json['command']['node'])
+        entry.data = json['command']['data']
+        return entry
+    if json['command']['type'] == 'save_configuration':
+        entry = SaveConfigurationEntry()
+        entry.delay = json['delay']
+        entry.node = ldf.get_slave(json['command']['node'])
+        return entry
+    if json['command']['type'] == 'assign_frame_id':
+        entry = AssignFrameIdEntry()
+        entry.delay = json['delay']
+        entry.node = ldf.get_slave(json['command']['node'])
+        entry.frame = ldf.get_frame(json['command']['frame'])
+        return entry
+    if json['command']['type'] == 'unassign_frame_id':
+        entry = UnassignFrameIdEntry()
+        entry.delay = json['delay']
+        entry.node = ldf.get_slave(json['command']['node'])
+        entry.frame = ldf.get_frame(json['command']['frame'])
+        return entry
+    if json['command']['type'] == 'assign_frame_id_range':
+        entry = AssignFrameIdRangeEntry()
+        entry.delay = json['delay']
+        entry.node = ldf.get_slave(json['command']['node'])
+        entry.frame_index = json['command']['frame_index']
+        entry.pids = json['command']['pids']
+        return entry
+    if json['command']['type'] == 'free_format':
+        entry = FreeFormatEntry()
+        entry.delay = json['delay']
+        entry.data = json['command']['data']
+        return entry
+    raise ValueError("Unknown schedule command type")
+
+def _populate_schedule_tables(json: dict, ldf: LDF):
+    if 'schedule_tables' in json:
+        for table in json['schedule_tables']:
+            schedule_table = ScheduleTable(table['name'])
+            for item in table['schedule']:
+                entry = _create_schedule_table_entry(item, ldf)
+                schedule_table.schedule.append(entry)
+            ldf._schedule_tables[schedule_table.name] = schedule_table
+
 def _link_ldf_signals(json: dict, ldf: LDF):  # noqa: C901
     for signal in _require_key(json, 'signals', 'LDF missing Signals section.'):
         signal_obj = ldf.get_signal(signal['name'])
@@ -211,6 +320,13 @@ def _link_ldf_frames(json: dict, ldf: LDF):
             slave.publishes_frames.append(frame_obj)
             frame_obj.publisher = slave
 
+def _link_ldf_schedule_table(json: dict, ldf: LDF):
+    if "event_triggered_frames" not in json:
+        return
+    for frame in json['event_triggered_frames']:
+        frame_obj = ldf.get_event_triggered_frame(frame['name'])
+        frame_obj.collision_resolving_schedule_table = ldf.get_schedule_table(frame['collision_resolving_schedule_table'])
+
 def _populate_ldf_encoding_types(json: dict, ldf: LDF):
     if json.get('signal_encoding_types') is None or json.get('signal_representations') is None:
         return
@@ -221,7 +337,9 @@ def _populate_ldf_encoding_types(json: dict, ldf: LDF):
         ldf._signal_encoding_types[encoding_type['name']] = LinSignalEncodingType(encoding_type['name'], converters)
     for representations in json['signal_representations']:
         for signal in representations['signals']:
-            ldf._signal_representations[ldf.get_signal(signal)] = ldf._signal_encoding_types[representations['encoding']]
+            signal_obj = ldf.get_signal(signal)
+            signal_obj.encoding_type = ldf._signal_encoding_types[representations['encoding']]
+            ldf._signal_representations[signal_obj] = ldf._signal_encoding_types[representations['encoding']]
 
 def _convert_encoding_value(json: dict) -> ValueConverter:
     if json['type'] == 'logical':
@@ -324,7 +442,10 @@ class LDFTransformer(Transformer):
         return tree[0:]
 
     def diagnostic_signals(self, tree):
-        return ("diagnostic_signals", [])
+        return ("diagnostic_signals", tree)
+
+    def diagnostic_signal_definition(self, tree):
+        return {"name": tree[0], "width": int(tree[1]), "init_value": tree[2]}
 
     def frames(self, tree):
         return ("frames", tree)
@@ -354,7 +475,13 @@ class LDFTransformer(Transformer):
         return tree[0:]
 
     def diagnostic_frames(self, tree):
-        return ("diagnostic_frames", [])
+        return ("diagnostic_frames", tree)
+
+    def diagnostic_frame_definition(self, tree):
+        return {"name": tree[0], "frame_id": int(tree[1]), "signals": tree[2]}
+
+    def diagnostic_frame_signals(self, tree):
+        return tree[0:]
 
     def node_attributes(self, tree):
         return ("node_attributes", tree[0:])
@@ -415,7 +542,7 @@ class LDFTransformer(Transformer):
         return {"name": tree[0], "schedule": tree[1:]}
 
     def schedule_table_entry(self, tree):
-        return {"command": tree[0], "delay": tree[1]}
+        return {"command": tree[0], "delay": tree[1] * 0.001}
 
     def schedule_table_command(self, tree):
         return tree[0]
